DROP SEQUENCE test_upd_sq;
DROP TABLE test_upd PURGE;

CREATE SEQUENCE test_upd_sq;

CREATE TABLE test_upd (
 ID INTEGER NOT NULL
,NAME VARCHAR2(100)
,mod_by VARCHAR2(100)
,mod_dt DATE
)
/
--------------------------------------------------------------------------------
-- No instrumentation
--------------------------------------------------------------------------------
INSERT INTO test_upd SELECT test_upd_sq.nextval, object_name, 'test', SYSDATE
  FROM all_objects
  WHERE ROWNUM <= 100;
COMMIT;

CREATE OR REPLACE PACKAGE table_api
AS
PROCEDURE upd_all_initcap;
END table_api;
/
CREATE OR REPLACE PACKAGE BODY table_api
AS

PROCEDURE upd_all_initcap IS
   l_loop_idx      NUMBER := 0;
BEGIN
   dbms_lock.sleep(8); -- pretend that a bunch of processing is being done
   FOR lr IN (SELECT * FROM test_upd) LOOP
      l_loop_idx := l_loop_idx + 1;
      dbms_output.put_line('About to update row# '||lr.id);
      UPDATE test_upd
         SET NAME   = INITCAP(NAME)
            ,mod_by = SYS_CONTEXT('userenv', 'client_identifier')
            ,mod_dt = SYSDATE
       WHERE ID = lr.id;
   END LOOP;
   COMMIT;

END upd_all_initcap;

END table_api;
/

PROMPT Calling table_api.upd_all_initcap. Wait for DBMS_OUTPUT to be retrieved from buffer...
SET SERVEROUTPUT ON SIZE 1000000
BEGIN
   table_api.upd_all_initcap;
END;
/

--------------------------------------------------------------------------------
TRUNCATE TABLE test_upd;

INSERT INTO test_upd SELECT test_upd_sq.nextval, object_name, NULL, NULL
  FROM all_objects
  CROSS JOIN (SELECT LEVEL AS num FROM dual CONNECT BY LEVEL <= 4) lvl;
COMMIT;

CREATE OR REPLACE PACKAGE table_api
AS
PROCEDURE upd_all_initcap;
END table_api;
/
CREATE OR REPLACE PACKAGE BODY table_api
AS

PROCEDURE upd_all_initcap IS
   l_longops_rowid PLS_INTEGER := -1;
   l_longops_num   PLS_INTEGER;
   l_total_rows    INTEGER := 0;
   l_loop_idx      NUMBER := 0;
BEGIN
   -- pretend this is being done by the client application or at the top of the batch job
   -- I only include this here to demonstrate action and client_info changing over time
   dbms_application_info.set_module(module_name => 'table_api.upd_all_initcap'
                                   ,action_name => 'set client_id');
   dbms_application_info.set_client_info(client_info => 'Bill the Cat');
   dbms_lock.sleep(5); -- pause in order to switch window to query on v$session
   dbms_session.set_identifier('Bill the Cat');

   dbms_application_info.set_client_info(client_info => 'get TEST_UPD size');
   SELECT COUNT(*)
     INTO l_total_rows -- needed for longops tracking
     FROM test_upd;

   dbms_application_info.set_action(action_name => 'loop TEST_UPD and update to initcap');
   FOR lr IN (SELECT * FROM test_upd) LOOP
      l_loop_idx := l_loop_idx + 1;
      dbms_application_info.set_client_info(client_info => 'updating ' || lr.id);
      UPDATE test_upd
         SET NAME   = INITCAP(NAME)
            ,mod_by = SYS_CONTEXT('userenv', 'client_identifier')
            ,mod_dt = SYSDATE
       WHERE ID = lr.id;
      dbms_application_info.set_session_longops(
       rindex      => l_longops_rowid -- first call should be -1, afterwards holds rowid of v
      ,slno        => l_longops_num -- used interally to communicate info between calls
      ,CONTEXT     => lr.id -- can be anything, we'll use it to store ID being processed
      ,op_name     => 'table_api.upd_all_initcap' -- the operation, limited to 64 chars
      ,target_desc => 'TEST_UPD' -- the target object of the DDL or DML statement
      ,totalwork   => l_total_rows -- total work you know the process must complete
      ,sofar       => l_loop_idx -- amount of work done so far
      ,units       => 'rows updated.'); -- units of measure for the values in totalwork and sofar
   END LOOP;
   dbms_application_info.set_client_info(client_info => NULL); -- cleanup client_info

   dbms_application_info.set_action(action_name => 'committing changes');
   COMMIT;
   dbms_lock.sleep(5); -- pause to observe effects in v$session
   dbms_application_info.set_module(module_name => NULL, action_name => NULL); -- cleanup the rest

END upd_all_initcap;

END table_api;
/

PROMPT Calling table_api.upd_all_initcap. Switch to another window and query V$SESSION, V$SESSION_LONGOPS, V$SQL, and V$SQLAREA
SET SERVEROUTPUT ON SIZE 1000000 
BEGIN
   table_api.upd_all_initcap;
END;
/

--------------------------------------------------------------------------------
-- API instrumentation
--------------------------------------------------------------------------------
TRUNCATE TABLE test_upd;

INSERT INTO test_upd SELECT test_upd_sq.nextval, object_name, NULL, NULL
  FROM all_objects
  CROSS JOIN (SELECT LEVEL AS num FROM dual CONNECT BY LEVEL <= 5) lvl;
COMMIT;

CREATE OR REPLACE PACKAGE table_api
AS
PROCEDURE upd_all_initcap;
END table_api;
/
CREATE OR REPLACE PACKAGE BODY table_api
AS

PROCEDURE upd_all_initcap IS
   l_longop        env.t_longop;
BEGIN
   env.tag(i_info => 'set client ID');
   env.init_client_ctx('Bill the Cat');
   -- pretend this is being done by the client application or at the top of the batch job
   -- I only include this here to demonstrate action and client_info changing over time
   dbms_lock.sleep(5);

   env.tag(i_info => 'get TEST_UPD size');
   SELECT COUNT(*)
     INTO l_longop.total_work -- needed for longops tracking
     FROM test_upd;
   dbms_lock.sleep(5);

   env.tag(i_info => 'loop TEST_UPD and update to initcap');
   l_longop.op_nm := 'loop and update initcap';
   l_longop.units_of_measure := 'rows updated.';
   l_longop.work_target := 'TEST_UPD';
   env.tag_longop(l_longop); -- seed row, then do the operation

   FOR lr IN (SELECT * FROM test_upd) LOOP
      l_longop.work_done := l_longop.work_done + 1;
      env.tag(i_info => 'updating ' || lr.id);
      UPDATE test_upd
         SET NAME   = INITCAP(NAME)
            ,mod_by = SYS_CONTEXT('userenv', 'client_identifier')
            ,mod_dt = SYSDATE
       WHERE ID = lr.id;
      env.tag_longop(l_longop);
   END LOOP;
   env.untag; -- cleanup client_info

   env.tag(i_info => 'committing changes');
   COMMIT;
   dbms_lock.sleep(5);
   env.untag; -- cleanup the rest

END upd_all_initcap;

END table_api;
/

PROMPT Calling table_api.upd_all_initcap. Switch to another window and query V$SESSION and V$SESSION_LONGOPS
BEGIN
   table_api.upd_all_initcap;
END;
/

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--PROMPT Filling TEST_UPD with 3M rows in hopes of seeing longops data automatically...
--TRUNCATE TABLE test_upd;
--SET TIMING ON
--INSERT INTO test_upd SELECT test_upd_sq.nextval, object_name, NULL, NULL
--  FROM all_objects
--  CROSS JOIN (SELECT LEVEL AS num FROM dual CONNECT BY LEVEL <= 400) lvl;
--COMMIT;
--BEGIN
--   ddl_utils.analyze_table('TEST_UPD');
--END;
--/
--PROMPT Updating 3M rows in TEST_UPD...
--BEGIN
--   table_api.upd_all_rows;
--END;
--/

-- SQL statements to peek into performance views
SELECT * FROM v$sql vs JOIN v$session s ON s.sql_id = vs.sql_id WHERE s.client_identifier = 'Bill the Cat';
 
SELECT * FROM v$sqltext st JOIN v$session s ON s.sql_id = st.sql_id WHERE s.client_identifier = 'Bill the Cat' ORDER BY piece;

SELECT * FROM v$sqlarea sq JOIN v$session s ON s.sql_id = sq.sql_id WHERE s.client_identifier = 'Bill the Cat';

SELECT SID, serial#, username, taddr, lockwait, client_identifier, module, action, client_info, osuser, machine, program, sql_id, wait_class, seconds_in_wait, state
  FROM v$session
 WHERE client_identifier = 'Bill the Cat';

SELECT s.username, s.client_identifier, si.sid, si.block_gets, si.consistent_gets, si.physical_reads, si.block_changes, si.consistent_changes
  FROM v$sess_io si JOIN v$session s ON s.sid = si.sid
 WHERE s.client_identifier = 'Bill the Cat';

SELECT s.module, s.action, s.client_info, s.client_identifier, lo.*
  FROM v$session_longops lo
  JOIN v$session s ON s.sid = lo.sid AND s.serial# = lo.SERIAL#
 WHERE time_remaining > 0
   AND (lo.sid, lo.serial#) = (SELECT SID, serial# FROM v$session WHERE client_identifier = 'Bill the Cat');

-- Progress Bar query
SELECT s.module, s.action, s.client_info, s.client_identifier
      ,start_time AS "Started"
      ,'Operation: '||opname||' on '||NVL(target,NVL(target_desc,'--not given--'))||' is '||
       'Roughly ' || ROUND((((time_remaining+elapsed_seconds) - time_remaining) / (time_remaining+elapsed_seconds)) * 100) ||'% Complete' AS "Progress"
      ,ROUND((time_remaining/60),1)||' minutes' AS "Remaining"
      ,vsql.sql_text
  FROM v$session_longops l
  JOIN v$session s ON l.sid = s.sid AND l.serial# = s.serial#
  JOIN v$sqltext vsql ON vsql.hash_value = s.sql_hash_value AND vsql.piece = 0
 WHERE s.client_identifier = 'Bill the Cat'
   AND l.time_remaining > 0
 ORDER BY start_time DESC;
